"""
Custodian is a simple, robust and flexible just-in-time (JIT) job management
framework written in Python.
"""

__author__ = "Shyue Ping Ong, William Davidson Richards, Stephen Dacek, Xiaohui Qu, Matthew Horton, Samuel M. Blau"
__version__ = "2023.5.7"

from .custodian import Custodian
